#!/usr/bin/env python



###                                           ###
#                                               #
# A utility class imported by patch_conf.py     #
#                                               #
# Author: marko dot obrovac monkey inria dot fr #
#                                               #
###                                           ###



import os;
import shutil;



class FileConf:
	'''
	Represents the configuration file to be read
	and whose configuration should be changed
	'''
	
	
	__data = []
	__fname = ''
	__params = {}
	__separator = '='
	__modified = False
	__whole_lines = False
	__verbose = False
	__backup_dir = ''
	__backup = True
	
	
	def __init__( self, filename, conf_separator = '=', talk = False, compare_whole_lines = False, bckp_dir = '', save_bckp = True, filecontents = None ):
		
		self.__fname = filename
		self.__separator = conf_separator;
		if self.__separator == '\t':
			self.__separator = ' '
		self.__verbose = talk
		self.__whole_lines = compare_whole_lines
		self.__backup_dir = bckp_dir
		if len( self.__backup_dir ) > 0 and not self.__backup_dir.endswith( '/' ):
			self.__backup_dir += '/'
		self.__backup = save_bckp
		if isinstance( filecontents, list ):
			self.__data = filecontents
			self.parse()
		else:
			self.load_data()
	
	
	def __speak( self, what ):
		
		if not self.__verbose:
			return

		line = ''

		if len( self.__fname ) > 0:
			line = '| ' + self.__fname + ' | '

		print line + what
	
	
	def load_data( self, filename = '' ):
		
		if len( filename ) == 0:
			filename = self.__fname
		else:
			self.__fname = filename
		self.__modified = False
		self.__data = []
		if not os.path.isfile( filename ):
			return False
		f = open( filename, 'r' )
		self.__data = f.readlines()
		f.close()
		return self.parse()
	
	
	def parse( self ):
		
		self.__params = {}
		if self.__whole_lines:
			self.__speak( 'Parsing whole lines' )
			return True
		for index, line in enumerate( self.__data ):
			line = line.expandtabs( 1 ).strip()
			if len( line ) == 0 or line.startswith( '#' ):
				continue
			parts = line.split( self.__separator, 1 )
			if len( parts ) <= 1:
				continue
			self.__params[parts[0].lower().strip()] = { 'index' : index, 'orig_name' : parts[0].strip(), 'value' : parts[1].strip(), 'modified' : False }
		
		return True
	
	
	def append_line( self, line ):
		
		if not self.__whole_lines:
			return False

		for ll in self.__data:
			if ll.strip() == line.strip():
				return False
		
		self.__speak( 'Appending new line: "' + line.strip()  + '"' )
		if line[-1] != '\n':
			line += '\n'
		self.__data.append( line )
		self.__modified = True
		return True
	
	
	def alter_param( self, param_name, param_value = None, add_nonexistent = True ):
		
		if self.__whole_lines:
			return self.append_line( param_name )

		param_name_lower = param_name.lower()
		param_value = param_value.expandtabs( 1 ).strip()
		if ( param_name_lower not in self.__params ):
			if not add_nonexistent:
				return False
			self.__params[param_name_lower] = { 'index' : -1, 'orig_name' : param_name, 'value' : param_value, 'modified' : True }
			self.__modified = True
			self.__speak( 'Adding param: ' + param_name + ' ( "' + param_value + '" )' )
			return True
		if param_value != self.__params[param_name_lower]['value']:
			self.__speak( 'Modifying param ' + param_name + ' from "' + self.__params[param_name_lower]['value'] + '" to "' + param_value  + '"' )
			self.__params[param_name_lower]['value'] = param_value
			self.__params[param_name_lower]['modified'] = True
			self.__modified = True
			return True
		return False
	
	
	def write( self ):
		
		if not self.__modified:
			self.__speak( 'No modifications were made' )
			return True

		backup_path = self.__fname + '.cxos.bkp'
		diff_path = self.__fname + '.cxos.diff'
		
		if not self.__whole_lines:
			for key, val in self.__params.iteritems():
				if not val['modified']:
					continue
				confline = val['orig_name'] + self.__separator + val['value'] + '\n'
				if val['index'] == -1:
					self.__data.append( confline )
				else:
					self.__data[val['index']] = confline
		
		if os.path.isfile( self.__fname ) and self.__backup:
			if len( self.__backup_dir ) > 0:
				tmp_fname = self.__fname.replace( '/', '_' )
				if tmp_fname.startswith( '._' ):
					tmp_fname = tmp_fname[2:]
				elif tmp_fname.startswith( '_' ):
					tmp_fname = tmp_fname[1:]
				backup_path = self.__backup_dir + tmp_fname;
				diff_path = self.__backup_dir + tmp_fname + '.diff';
			self.__speak( 'Saving backup to ' + backup_path )
			shutil.move( self.__fname, backup_path )
		
		self.__speak( 'Saving file ...' )
		f = open( self.__fname, 'w' )
		for line in self.__data:
			f.write( line )
		f.close()
		
		if os.path.isfile( self.__fname ) and self.__backup:
			self.__speak( 'Saving differences to ' + diff_path )
			os.system( 'diff -u ' + backup_path + ' ' + self.__fname + ' > ' + diff_path )
		
		return True
	
	
	def get_params( self ):
		
		if not self.__whole_lines:
			lst = []
			for prm in self.__params.keys():
				lst.append( self.__params[prm]['orig_name'] )
			return lst

		return self.__data
	
	
	def get_param_value( self, param_name ):
		
		if self.__whole_lines:
			return param_name

		pn_lower = param_name.strip().lower()
		if pn_lower in self.__params.keys():
			return self.__params[pn_lower]['value']
		return ''
	
	
	def __repr__( self ):
		
		return 'DATA:\n' + repr( self.__data ) + '\n\nPARAMS:\n' + repr( self.__params )

