#!/usr/bin/env python


###                                                      ###
#                                                          #
# A xosautoconfig utility script to resolve node types and #
# determine the services the node will run                 #
#                                                          #
# Author: marko dot obrovac monkey inria dot fr            #
#                                                          #
###                                                      ###


import sys
import os
import getopt


class NodeConf:
	'''
	A class to manipulate node type and services
	to be run on those nodes in XtreemOS
	'''


	__fname_node_types = ''
	__fname_services = ''

	__services = {}
	__nodes = {}

	__def_type = ''


	def __init__( self, node_types_file, services_file ):
		
		self.__fname_node_types = node_types_file
		self.__fname_services = services_file
		self.__load_data()
	
	
	def __load_data( self ):
		
		if not self.__load_services():
			return False

		if not self.__load_node_types():
			return False

		return True
	

	def __load_services( self ):
		
		if not os.path.isfile( self.__fname_services ):
			self.__services = {}
			return False

		f = open( self.__fname_services, 'r' )
		file_data = f.readlines()
		f.close()

		for line in file_data:
			line = line.expandtabs().strip()
			# ignore comments
			if len( line ) == 0 or line.startswith( '#' ):
				continue
			# split the line
			parts = line.split( ':', 2 )
			if len( parts ) != 2:
				continue
			services = parts[1].strip()
			# find out whether we have a node type or a hostname (fqhn)
			leftparts = parts[0].split( '.' )
			if len( leftparts ) > 1:
				# we're dealing with a hostname or ip
				hostname = parts[0].strip()
				if hostname in self.__nodes:
					if 'specific' in self.__nodes[hostname]:
						self.__nodes[hostname]['specific'] = self.__nodes[hostname]['specific'] + ' ' + services
					else:
						self.__nodes[hostname]['specific'] = services
				else:
					self.__nodes[hostname] = {'specific' : services }
			else:
				# we're dealing with a node type
				node_type = parts[0].strip()
				if node_type in self.__services:
					self.__services[node_type] = self.__services[node_type] + ' ' + services
				else:
					self.__services[node_type] = services

		return True
	
	
	def __load_node_types( self ):
		
		if not os.path.isfile( self.__fname_node_types ):
			self.__nodes = {}
			return False

		f = open( self.__fname_node_types, 'r' )
		file_data = f.readlines()
		f.close()

		for line in file_data:
			line = line.expandtabs().strip()
			# ignore comments
			if len( line ) == 0 or line.startswith( '#' ):
				continue
			# split the line
			parts = line.split( ':', 2 )
			if len( parts ) != 2:
				continue
			node_type = parts[0].strip()
			hostnames = parts[1].strip()
			if node_type == 'default-node-type':
				self.__def_type = hostnames
			else:
				hnames = hostnames.split( ' ' )
				for host in hnames:
					host = host.strip()
					if host in self.__nodes:
						self.__nodes[host][node_type] = True
					else:
						self.__nodes[host] = { node_type : True }

		return True
	
	
	def get_services( self, hostname ):
		
		services = ''
		
		if hostname in self.__nodes:
			for key in iter( self.__nodes[hostname] ):
				if key == 'specific':
					services = services + ' ' + self.__nodes[hostname][key]
				else:
					services = services + ' ' + self.__services[key]
		else:
			if self.__def_type in self.__services:
				services = self.__services[self.__def_type]
		
		if 'all-nodes' in self.__services:
			services = services + ' ' + self.__services['all-nodes']
		
		return services.strip()
	
	
	def __repr__( self ):
		
		return '*** NODES:\n' + repr( self.__nodes ) + '\n\n*** SERVICES:\n' + repr( self.__services ) + '\n\n*** DEF TYPE: ' + self.__def_type;


def print_err( msg ):
	
	print >> sys.stderr, msg


def usage():
	
	print_err( 'nodes.py is a utility script in the xosautoconfig package which' )
	print_err( 'determines the services that should be run on a given node. It is' )
	print_err( 'executed by the xosautoconfig script and it is not meant to be used' )
	print_err( 'as a stand-alone program, although it can be forced to do so.' )
	print_err( '' )
	print_err( 'Usage: nodes.py [OPTIONS] <hostname>' )
	print_err( '   <hostname>    the hostname of the node whose list of services' )
	print_err( '                 will be pulled out of the "nodeTypes" and "services"' )
	print_err( '                 files' )
	print_err( 'available OPTIONS are:' )
	print_err( '   -d, --datapath=path  the path to the directory containing the' )
	print_err( '                        files of interest ("nodeTypes" and "services")' )
	print_err( '                        NOTE: if an invalid path is specified, the' )
	print_err( '                        script will return an empty list and silently' )
	print_err( '                        exit (i.e. no error will occur)' )
	print_err( '   -f, --force          allow the script to operate as a stand-alone' )
	print_err( '                        program (not recommended)' )
	print_err( '   -h, --help           print this help screen and exit' )
	print_err( '' )


def main():
	
	dpath = ''
	force = False
	
	try:
		opts, args = getopt.gnu_getopt( sys.argv[1:], 'd:fh', ['datapath=', 'force', 'help'] )
	except getopt.GetoptError, err:
		print str( err )
		usage()
		sys.exit( 2 )
	
	for op, ar in opts:
		if op in ( '-d', '--datapath' ):
			if len( ar ) > 0:
				if ar.endswith( '/' ):
					dpath = ar
				else:
					dpath = ar + '/'
		elif op in ( '-f', '--force' ):
			force = True
		elif op in ( '-h', '--help' ):
			usage()
			sys.exit( 0 )
		else:
			print_err( op + ': argument not allowed! Use --help to find out more. Leaving...' )
			sys.exit( 3 )
	
	if 'CONFIGXOS_ACTIVE' not in os.environ.keys() and not force:
		print_err( 'This script can be run only from the xosautoconfig tool!' )
		print_err( 'Use --help to find out more' )
		sys.exit( 1 )
	
	if len( args ) != 1:
		print_err( 'Wrong number of arguments!' )
		print_err( 'Use --help to find out more about the syntax. Leaving...' )
		sys.exit( 4 )

	nc = NodeConf( dpath + 'nodeTypes', dpath + 'services' )
	print nc.get_services( args[0] )


if __name__ == '__main__':
	main()
	sys.exit( 0 )
