#!/usr/bin/env python


###                                                         ###
#                                                             #
# A xosautoconfig utility script to patch configuration files #
#                                                             #
# Author: marko dot obrovac monkey inria dot fr               #
#                                                             #
###                                                         ###


import sys
import os
import getopt
import fileconf



def patch_conf( param_list, dest_file, p_separator, p_verbose, p_compare_whole_lines, p_bckp_dir, p_save_bckp ):
	
	if len( param_list ) == 0:
		if p_verbose:
			print 'Nothing to patch in ' + dest_file
		return
	
	print 'Patching data in "' + dest_file + '" with params: ',
	for param in param_list:
		print param['name'] + ', ',
	print ''

	fc_dest = fileconf.FileConf( dest_file, p_separator, p_verbose, p_compare_whole_lines, p_bckp_dir, p_save_bckp )

	for param in param_list:
		fc_dest.alter_param( param['name'], param['value'] )
	
	fc_dest.write()


def patch_conf_with_file( src_file, dest_file, p_separator, p_verbose, p_compare_whole_lines, p_bckp_dir, p_save_bckp ):
	
	fc_src = fileconf.FileConf( src_file, p_separator, p_verbose, p_compare_whole_lines )
	fc_dest = fileconf.FileConf( dest_file, p_separator, p_verbose, p_compare_whole_lines, p_bckp_dir, p_save_bckp )
	
	if p_verbose:
		print 'Patching data in "' + dest_file + '" with data in "' + src_file + '"'

	for param in fc_src.get_params():
		fc_dest.alter_param( param, fc_src.get_param_value( param ) )
	
	fc_dest.write()


def usage():
	
	print 'patch_conf.py is a utility script in the xosautoconfig package, used to patch'
	print 'different xos configuration files. It is called by the xosautoconfig script and'
	print 'is not meant to be used as a stand-alone program, although you can force it'
	print 'to do so (use it at your own risk!)'
	print ''
	print 'Usage: patch_conf.py [OPTIONS] <file_to_patch>'
	print 'available OPTIONS are:'
	print '   -l, --wholelines           treat whole lines as parameter arguments'
	print '   -i, --input=filename       the file whose data will be used to patch the'
	print '                              target configuration file (<file_to_patch>)'
	print '   -s, --separator=sep_char   use the specified key/value separator character'
	print '   -p, --params=param_names   specify parameter names devided by a coma'
	print '   -v, --values=param_values  specify parameter values, devided by a coma'
	print '                              (the number of values must be equal ot greater'
	print '                              than the number of parameter names)'
	print '   -q, --quiet                supress messages'
	print '   -d, --backup-dir           specify the directory in which to save the backup'
	print '   -b, --nobackup             do not make a backup of the file being changed'
	print '   -f, --force                allow the script to operate as a stand-alone'
	print '                              program (not recommended - use at your own risk)'
	print '   -h, --help                 print this help screen and exit'
	print ''


def main():
	
	separator = '='
	verbose = True
	whole_lines = False
	input_fname = ''
	params = []
	backup_dir = ''
	save_backup = True
	force = False
	
	try:
		opts, args = getopt.gnu_getopt( sys.argv[1:], 'li:s:p:v:qd:bfh', ['wholelines', 'input=', 'separator=', 'params=', 'values=', 'quiet', 'backup-dir=', 'nobackup', 'force', 'help'] )
	except getopt.GetoptError, err:
		print str( err )
		usage()
		sys.exit( 2 )
	
	for op, ar in opts:
		if op in ( '-l', '--wholelines' ):
			whole_lines = True
		elif op in ( '-i', '--input' ):
			if not os.path.isfile( ar ):
				print ar + ': file not found, nothing to do here! Leaving...'
				sys.exit( 1 )
			input_fname = ar
		elif op in ( '-s', '--separator' ):
			separator = ar[0];
		elif op in ( '-p', '--params' ):
			prs = ar.split( ',' )
			for prm in prs:
				params.append( { 'name' : prm, 'value' : '' } )
		elif op in ( '-v', '--values' ):
			if len( params ) == 0:
				print "You must specify the parameter names before the values! Leaving..."
				sys.exit( 3 )
			vls = ar.split( ',' )
			if len( vls ) > len( params ):
				print 'WARNING: the following values will be ignored: ' + repr( vls[len( params ):] )
			for vi, vl in enumerate( vls ):
				if vi >= len( params ):
					break
				params[vi]['value'] = vl
		elif op in ( '-q', '--quiet' ):
			verbose = False
		elif op in ( '-d', '--backup-dir' ):
			backup_dir = ar
		elif op in ( '-b', '--nobackup' ):
			save_backup = False
		elif op in ( '-f', '--force' ):
			force = True
		elif op in ( '-h', '--help' ):
			usage()
			sys.exit( 0 )
		else:
			print op + ': argument not allowed! Leaving...'
			sys.exit( 4 )
	
	if 'CONFIGXOS_ACTIVE' not in os.environ.keys() and not force:
		print 'This script can be run only from the xosautoconfig tool!'
		print 'Use --help to find out more'
		sys.exit( 5 )
	
	if len( args ) == 0:
		print 'No file to patch! Use --help to find out more about the syntax. Leaving...'
		sys.exit( 6 )
	
	if len( args ) > 1:
		print 'Not all the command line arguments were consumed!'
		print 'Arguments left out: ' + repr( args )
		print 'Use --help to find out more about the syntax. Leaving...'
		sys.exit( 7 )
	
	if len( input_fname ) > 0:
		patch_conf_with_file( input_fname, args[0], separator, verbose, whole_lines, backup_dir, save_backup )
	else:
		patch_conf( params, args[0], separator, verbose, whole_lines, backup_dir, save_backup )


if __name__ == '__main__':
	main()
	sys.exit( 0 )
